<?php

namespace AuthStack\SP\Errors;

/**
 * Class InvalidStatusException
 * @package AuthStack\SP\Errors
 *
 *          Thrown when SAML Status isn't success
 */
class InvalidStatusException extends \Exception
{}